#-*-coding:mbcs-*- 
#----------------------------------------------------------------------------
# Project Name: MOSS for Windows
# File Name: moss-gui.py
# Description: MOSS GUI Interface.
#
# Created: 2004. 10. 11
# RCS-ID: $Id: moss-gui.py,v 1.22 2004/10/22 01:30:15 myunggoni Exp $
# Copyright: (c) 2004 by myunggoni
# License: GNU General Public License
# Author: Myung-Gon Park <myunggoni@users.kldp.net>
#----------------------------------------------------------------------------

__package__ = "MOSS for Windows"
__version__ = "0.1.1"

import sys
import os
import re
import string
import time
import socket
import threading
import ConfigParser
import moss

from cjkcodecs import cp949, aliases

if hasattr(sys, "setdefaultencoding"):
	sys.setdefaultencoding("cp949")

import wx
import wx.lib.mixins.listctrl as listmix
import wx.lib.intctrl
import wx.lib.newevent

#----------------------------------------------------------------------------

ID_ABOUT = wx.NewId()
ID_EXIT = wx.NewId()
ID_SHOW = wx.NewId()
ID_HIDE = wx.NewId()

(UpdateConnectionEvent, EVT_UPDATE_CONNECTION) = wx.lib.newevent.NewEvent()

#----------------------------------------------------------------------------

class MyServer(threading.Thread):
	def __init__(self, parent):
		threading.Thread.__init__(self)
		self.frame = parent
		self.running = False
		self.config_file = self.frame.config_file
		self.server_mode = self.frame.server_mode
		self.hostname = self.frame.hostname
		self.port = self.frame.port
		self.server = moss.MOSS(self.config_file, self.server_mode)

	def IsRunning(self):
		return self.running

	def run(self):
		self.running = True

		try:
			self.server.run()
		except:
			self.running = False

			if self.running:
				event = UpdateConnectionEvent()
				wx.PostEvent(self.frame.panel, event)
    	
	def stop(self):
		self.running = False
		self.server.stop()

#----------------------------------------------------------------------------

class MyListCtrl(wx.ListCtrl, listmix.ListCtrlAutoWidthMixin):
	def __init__(self, parent, id, pos = wx.DefaultPosition,
				 size = wx.DefaultSize, style = wx.LC_REPORT
								 | wx.BORDER_NONE
								 | wx.LC_EDIT_LABELS
                                                                 | wx.LC_SINGLE_SEL
								 | wx.LC_SORT_ASCENDING):
		wx.ListCtrl.__init__(self, parent, id, pos, size, style)
		listmix.ListCtrlAutoWidthMixin.__init__(self)

		self.frame = parent.frame
		self.currentItem = None
		
		self.PopulateList(self.frame.dirs)
		
		self.Bind(wx.EVT_LIST_BEGIN_LABEL_EDIT, self.OnBeginEdit)
		self.Bind(wx.EVT_LIST_END_LABEL_EDIT, self.OnEndEdit)
		self.Bind(wx.EVT_LIST_ITEM_SELECTED, self.OnItemSelected)
		self.Bind(wx.EVT_COMMAND_RIGHT_CLICK, self.OnRightClick)
		self.Bind(wx.EVT_RIGHT_UP, self.OnRightClick)
		self.Bind(wx.EVT_RIGHT_DOWN, self.OnRightDown)

	def OnBeginEdit(self, event):
		event.Allow()

	def OnEndEdit(self, event):
		label = event.GetText()

		if label:
			self.dirs[self.GetItemData(self.currentItem)][0] = label
			self.SetColumnWidth(0, wx.LIST_AUTOSIZE)
			self.SetColumnWidth(1, wx.LIST_AUTOSIZE)

	def OnRightDown(self, event):
		self.x = event.GetX()
		self.y = event.GetY()
		item, flags = self.HitTest((self.x, self.y))

		if flags & wx.LIST_HITTEST_ONITEM:
			self.Select(item)
			self.selected = True
		else:
			self.selected = False

		event.Skip()

	def OnRightClick(self, event):
	        if not hasattr(self, "popupEditLabel"):
			self.popupEditLabel = wx.NewId()
			self.popupEditDir = wx.NewId()
			self.popupRemoveSource = wx.NewId()
                        self.popupAddSource = wx.NewId()
		
			self.Bind(wx.EVT_MENU, self.OnPopupEditLabel, id = self.popupEditLabel)
			self.Bind(wx.EVT_MENU, self.OnPopupEditDir, id = self.popupEditDir)
			self.Bind(wx.EVT_MENU, self.OnPopupRemoveSource, id = self.popupRemoveSource)                        
			self.Bind(wx.EVT_MENU, self.OnPopupAddSource, id = self.popupAddSource)

		menu = wx.Menu()

		if self.selected:
			menu.Append(self.popupEditLabel, "Edit Label")
			menu.Append(self.popupEditDir, "Edit Directory")
			menu.Append(self.popupRemoveSource, "Remove Source")
		else:
			menu.Append(self.popupAddSource, "Add Source")

		self.PopupMenu(menu, (self.x, self.y))
		menu.Destroy()

	def OnPopupEditLabel(self, event):
		self.EditLabel(self.currentItem)
		
	def OnPopupEditDir(self, event):
		dialog = wx.DirDialog(self, "Source Directory",
                          style = wx.DD_DEFAULT_STYLE)

		if dialog.ShowModal() == wx.ID_OK:
			path = dialog.GetPath()
			self.SetStringItem(self.currentItem, 1, path)
			self.dirs[self.GetItemData(self.currentItem)][1] = path
			self.SetColumnWidth(0, wx.LIST_AUTOSIZE)
			self.SetColumnWidth(1, wx.LIST_AUTOSIZE)

		dialog.Destroy()

	def OnPopupRemoveSource(self, event):
		del self.dirs[self.GetItemData(self.currentItem)]
		self.PopulateList(self.dirs, True)
		
	def OnPopupAddSource(self, event):
		dialog = AddSource(self, -1, "Add Source")	

		if dialog.ShowModal() == wx.ID_OK:
			value = dialog.GetValue()
			self.dirs.append(value)
			self.PopulateList(self.dirs, True)

		dialog.Destroy()

	def OnItemSelected(self, event):
		self.currentItem = event.m_itemIndex
		event.Skip()

	def PopulateList(self, dirs, renew = False):
		if not renew:
			self.InsertColumn(0, "Label")
			self.InsertColumn(1, "Directory")

			self.dirs = []

			for i in range(len(dirs)):
				dir = map(string.strip, string.split(dirs[i][1], '::'))
			
				if len(dir) == 1:
					label = os.path.basename(dir[0])
				else:
					label = dir[1]

				self.dirs.append([label, os.path.realpath(dir[0])])

		self.DeleteAllItems()

		for key in range(len(self.dirs)):
			index = self.InsertStringItem(sys.maxint, self.dirs[key][0])
			self.SetStringItem(index, 1, self.dirs[key][1])
			self.SetItemData(index, key)

		if self.dirs:
			self.SetColumnWidth(0, wx.LIST_AUTOSIZE)
			self.SetColumnWidth(1, wx.LIST_AUTOSIZE)

#----------------------------------------------------------------------------

class MyPanel(wx.Panel):
	def __init__(self, parent, id):
		wx.Panel.__init__(
			self, parent, id,
			style = wx.TAB_TRAVERSAL | wx.CLIP_CHILDREN | wx.NO_FULL_REPAINT_ON_RESIZE)
		self.frame = parent
		self.thread = None

		if not self.frame.server_mode in ("web", "live"):
			self.frame.server_mode = "web"
		
		self.serverModeCtrl = wx.ComboBox(self, -1, value = self.frame.server_mode, 
                                        choices = ["web", "live"],
                                        style = wx.CB_DROPDOWN | wx.CB_READONLY,
                                        size = (170, -1))
		
		if self.frame.hostname:
			self.hostnameCtrl = wx.TextCtrl(self, -1, self.frame.hostname, size = (170, -1))
		else:
			self.hostnameCtrl = wx.TextCtrl(self, -1, "localhost", size = (170, -1))

		self.portCtrl = wx.lib.intctrl.IntCtrl(self, -1, self.frame.port, size = (170, -1))
		self.advancedButton = wx.Button(self, -1, "Advanced...")
		self.list = MyListCtrl(self, -1, size = (-1, 130))
		self.saveButton = wx.Button(self, -1, "Save Preferences")
		self.startButton = wx.Button(self, -1, "Run Server")
		
		self.serverModeText = wx.StaticText(self, -1, "Server Mode", size = (100, -1), style = wx.ALIGN_LEFT)
		self.hostnameText = wx.StaticText(self, -1, "Hostname", size = (100, -1), style = wx.ALIGN_LEFT)
		self.portText = wx.StaticText(self, -1, "Port", size = (100, -1), style = wx.ALIGN_LEFT)
		self.sourcesText = wx.StaticText(self, -1, "Sources", size = (100, -1), style = wx.ALIGN_LEFT)

		self.generalBox = wx.StaticBox( self, -1, "Preferences")
		self.sizer = wx.BoxSizer(wx.VERTICAL)
		self.staticSizer = wx.StaticBoxSizer(self.generalBox, wx.VERTICAL)
		self.hSizer = wx.BoxSizer(wx.HORIZONTAL)

		self.hSizer.Add(self.serverModeText, 0, wx.ALIGN_CENTRE | wx.ALL, 3)
		self.hSizer.Add(self.serverModeCtrl, 0, wx.ALIGN_CENTRE | wx.TOP | wx.BOTTOM | wx.RIGHT, 3)
		
		self.staticSizer.Add(self.hSizer, 0, wx.ALIGN_LEFT | wx.ALL, 1)

		self.hSizer = wx.BoxSizer(wx.HORIZONTAL)
		self.hSizer.Add(self.hostnameText, 0, wx.ALIGN_CENTRE | wx.ALL, 3)
		self.hSizer.Add(self.hostnameCtrl, 0, wx.ALIGN_CENTRE | wx.TOP | wx.BOTTOM | wx.RIGHT, 3)

		self.staticSizer.Add(self.hSizer, 0, wx.ALIGN_LEFT | wx.BOTTOM | wx.LEFT | wx.RIGHT, 1)

		self.hSizer = wx.BoxSizer(wx.HORIZONTAL)
		self.hSizer.Add(self.portText, 0, wx.ALIGN_CENTRE | wx.ALL, 3)
		self.hSizer.Add(self.portCtrl, 0, wx.ALIGN_CENTRE | wx.TOP | wx.BOTTOM | wx.RIGHT, 3)

		self.staticSizer.Add(self.hSizer, 0, wx.ALIGN_LEFT | wx.BOTTOM | wx.LEFT | wx.RIGHT, 1)

		self.staticSizer.Add(self.advancedButton, 0, wx.ALIGN_RIGHT | wx.BOTTOM | wx.LEFT | wx.RIGHT, 4)
		self.staticSizer.Add((-1, 20), 0, wx.EXPAND)
		self.staticSizer.Add(self.sourcesText, 0, wx.ALIGN_LEFT | wx.ALL, 4)
		self.staticSizer.Add(self.list, 0, wx.EXPAND | wx.ALIGN_CENTRE | wx.BOTTOM | wx.LEFT | wx.RIGHT, 4)
		
		self.hSizer = wx.BoxSizer(wx.HORIZONTAL)
		self.hSizer.Add(self.saveButton, 0, wx.ALIGN_CENTRE | wx.TOP | wx.BOTTOM | wx.LEFT | wx.RIGHT, 4)
		self.hSizer.Add(self.startButton, 0, wx.ALIGN_CENTRE | wx.TOP | wx.BOTTOM | wx.RIGHT, 4)

		self.sizer.Add(self.staticSizer, wx.ALIGN_CENTRE | wx.ALL, 0)
		self.sizer.Add(self.hSizer, 0, wx.ALIGN_RIGHT | wx.ALL, 0)

		self.SetSizer(self.sizer)
		self.SetAutoLayout(True)
		
		self.Bind(wx.EVT_COMBOBOX, self.OnServerMode, self.serverModeCtrl)
		self.Bind(wx.EVT_TEXT, self.OnHostname, self.hostnameCtrl)
		self.Bind(wx.lib.intctrl.EVT_INT, self.OnPort, self.portCtrl)
		self.Bind(wx.EVT_BUTTON, self.OnAdvanced, self.advancedButton)
		self.Bind(wx.EVT_BUTTON, self.OnSavePreferences, self.saveButton)
		self.Bind(wx.EVT_BUTTON, self.OnRunServer, self.startButton)
	        self.Bind(EVT_UPDATE_CONNECTION, self.OnUpdateConnection)

	def OnServerMode(self, event):
		self.frame.server_mode = event.GetString()
		
	def OnHostname(self, event):
		self.frame.hostname = event.GetString()
		
	def OnPort(self, event):
		self.frame.port = event.GetValue()

	def OnAdvanced(self, event):
		dialog = Advanced(self, -1, "Advanced")	
		dialog.ShowModal()
		dialog.Destroy()

	def OnSavePreferences(self, event):
		self.DepopulateList(self.list.dirs)
		self.config = ConfigParser.ConfigParser()
		self.config.add_section("server")
		self.config.add_section("sources")
		self.config.add_section("acl")
		self.config.add_section("extra")

		self.config.set("server", "server-mode", self.frame.server_mode)
		self.config.set("server", "port", self.frame.port)
		self.config.set("server", "hostname", self.frame.hostname)
		self.config.set("server", "log", self.frame.log)
		self.config.set("server", "max-connection", self.frame.max_connection)
		self.config.set("server", "read-size", self.frame.read_size)
		self.config.set("server", "stream-type", self.frame.stream_type)
		self.config.set("server", "max-buffer", self.frame.max_buffer)
		self.config.set("server", "send-timeout", self.frame.send_timeout)
		self.config.set("server", "shuffle", self.frame.shuffle)
		self.config.set("server", "recursive", self.frame.recursive)
		self.config.set("server", "shuffle", self.frame.shuffle)

		for key in range(len(self.frame.dirs)):
			self.config.set("sources", 
				"source-dir%d" % self.frame.dirs[key][0], 
				self.frame.dirs[key][1])

		if self.frame.auth_table:
			self.config.set("acl", "auth-file", self.frame.auth_file)
		
			self.authFile = open(self.frame.auth_file, "w")

			for user, passwd in self.frame.auth_table.items():
				self.authFile.write("%s:%s\n" % (user, passwd))

			self.authFile.close()

		if self.frame.acls:
			acls = ""
			first = True

			for addr, masklen in self.frame.acls:
				addr = string.join([addr, masklen], "/")
		
				if first:
					acls += addr
					first = False
				else:
					acls += string.join([",", addr])

			self.config.set("acl", "allow", acls)
			
		self.config.set("extra", "template", self.frame.template)
		self.config.set("extra", "template-dir", self.frame.template_dir)

		self.configFile = open(self.frame.config_file, "w")
		self.config.write(self.configFile)
		self.configFile.close()
		
	def OnRunServer(self, event):
		self.DepopulateList(self.list.dirs)
		self.OnSavePreferences(None)

		if self.thread and self.thread.IsRunning():
			self.startButton.SetLabel("Run Server")
			self.frame.tbicon.SetIcon(self.frame.icon, __package__ + " - Stopped")
			self.thread.stop()
		else:
			self.startButton.SetLabel("Stop Server")
			self.frame.tbicon.SetIcon(self.frame.icon, __package__ + " - Started")
			self.thread = MyServer(self.frame)
			self.thread.start()

	def DepopulateList(self, dirs):
		self.frame.dirs = []

		for key in range(len(self.list.dirs)):
			self.frame.dirs.append((key + 1, 
				string.join([self.list.dirs[key][1],
				self.list.dirs[key][0]], " :: ")))		
					
	def OnUpdateConnection(self, event):
		self.startButton.SetLabel("Run Server")
		self.frame.tbicon.SetIcon(self.frame.icon, __package__ + " - Stopped")
		self.thread = None
		
#----------------------------------------------------------------------------

class AddAddress(wx.Dialog):
	def __init__(
			self, parent, id, title, size = wx.DefaultSize, pos = wx.DefaultPosition, 
			style = wx.DEFAULT_DIALOG_STYLE
			):
		pre = wx.PreDialog()
		pre.Create(parent, id, title, pos, size, style)

		self.PostCreate(pre)
		self.address = None
		
		self.addressText = wx.StaticText(self, -1, "Address", size = (80, -1))
		self.addressCtrl = wx.TextCtrl(self, -1, "", size = (150, -1))
		
		self.line = wx.StaticLine(self, -1, style = wx.LI_HORIZONTAL)
		self.button = wx.Button(self, wx.ID_OK, "OK")
		self.button.SetDefault()

		self.sizer = wx.BoxSizer(wx.VERTICAL)
		
		self.hSizer = wx.BoxSizer(wx.HORIZONTAL)
		self.hSizer.Add(self.addressText, 0, wx.ALIGN_CENTRE | wx.ALL, 4)
		self.hSizer.Add(self.addressCtrl, 0, wx.ALIGN_CENTRE | wx.BOTTOM | wx.TOP | wx.RIGHT, 4)
		self.sizer.Add(self.hSizer, 0, wx.ALIGN_CENTRE | wx.ALL, 1)

		self.sizer.Add(self.line, 0, wx.EXPAND | wx.ALIGN_CENTER_VERTICAL | wx.LEFT | wx.RIGHT, 5)
		self.sizer.Add(self.button, 0, wx.ALIGN_RIGHT | wx.ALL, 5)
		
		self.SetSizer(self.sizer)
		self.SetAutoLayout(True)
		self.sizer.Fit(self)

		self.Bind(wx.EVT_TEXT, self.OnAddress, self.addressCtrl)
		self.Bind(wx.EVT_BUTTON, self.OnButton, self.button, id = wx.ID_OK)

	def OnButton(self, event):
		if self.address:
			self.EndModal(wx.ID_OK)
		else:
			dialog = wx.MessageDialog(self, 'Please Enter Address', 'Warning',
				       wx.OK | wx.ICON_INFORMATION)
			dialog.ShowModal()
			dialog.Destroy()
		
	def OnAddress(self, event):
		self.address = event.GetString()

	def GetAddress(self):
		return self.address

#----------------------------------------------------------------------------

class AddUser(wx.Dialog):
	def __init__(
			self, parent, id, title, size = wx.DefaultSize, pos = wx.DefaultPosition, 
			style = wx.DEFAULT_DIALOG_STYLE
			):
		pre = wx.PreDialog()
		pre.Create(parent, id, title, pos, size, style)

		self.PostCreate(pre)
		self.username = None
		self.passwd = None
		self.passwd1 = None
		self.passwd2 = None
		
		self.usernameText = wx.StaticText(self, -1, "Username", size = (120, -1))
		self.passwdText1 = wx.StaticText(self, -1, "New Password", size = (120, -1))
		self.passwdText2 = wx.StaticText(self, -1, "Retype Password", size = (120, -1))
		self.usernameCtrl = wx.TextCtrl(self, -1, "", size = (150, -1))
		self.passwdCtrl1 = wx.TextCtrl(self, -1, "", size = (150, -1), style = wx.TE_PASSWORD)
		self.passwdCtrl2 = wx.TextCtrl(self, -1, "", size = (150, -1), style = wx.TE_PASSWORD)
		
		self.line = wx.StaticLine(self, -1, style = wx.LI_HORIZONTAL)
		self.button = wx.Button(self, wx.ID_OK, "OK")
		self.button.SetDefault()

		self.sizer = wx.BoxSizer(wx.VERTICAL)
		
		self.hSizer = wx.BoxSizer(wx.HORIZONTAL)
		self.hSizer.Add(self.usernameText, 0, wx.ALIGN_CENTRE | wx.ALL, 4)
		self.hSizer.Add(self.usernameCtrl, 0, wx.ALIGN_CENTRE | wx.BOTTOM | wx.LEFT | wx.RIGHT, 4)
		self.sizer.Add(self.hSizer, 0, wx.ALIGN_CENTRE | wx.ALL, 1)

		self.hSizer = wx.BoxSizer(wx.HORIZONTAL)
		self.hSizer.Add(self.passwdText1, 0, wx.ALIGN_CENTRE | wx.ALL, 4)
		self.hSizer.Add(self.passwdCtrl1, 0, wx.ALIGN_CENTRE | wx.BOTTOM | wx.LEFT | wx.RIGHT, 4)
		self.sizer.Add(self.hSizer, 0, wx.ALIGN_CENTRE | wx.ALL, 1)

		self.hSizer = wx.BoxSizer(wx.HORIZONTAL)
		self.hSizer.Add(self.passwdText2, 0, wx.ALIGN_CENTRE | wx.ALL, 4)
		self.hSizer.Add(self.passwdCtrl2, 0, wx.ALIGN_CENTRE | wx.BOTTOM | wx.LEFT | wx.RIGHT, 4)
		self.sizer.Add(self.hSizer, 0, wx.ALIGN_CENTRE | wx.ALL, 1)

		self.sizer.Add(self.line, 0, wx.EXPAND | wx.ALIGN_CENTER_VERTICAL | wx.LEFT | wx.RIGHT, 5)
		self.sizer.Add(self.button, 0, wx.ALIGN_RIGHT | wx.ALL, 5)
		
		self.SetSizer(self.sizer)
		self.SetAutoLayout(True)
		self.sizer.Fit(self)

		self.Bind(wx.EVT_TEXT, self.OnUsername, self.usernameCtrl)
		self.Bind(wx.EVT_TEXT, self.OnPasswd1, self.passwdCtrl1)
		self.Bind(wx.EVT_TEXT, self.OnPasswd2, self.passwdCtrl2)
		self.Bind(wx.EVT_BUTTON, self.OnButton, self.button, id = wx.ID_OK)

		
	def OnUsername(self, event):
		self.username = event.GetString()

	def OnPasswd1(self, event):
		self.passwd1 = event.GetString()

	def OnPasswd2(self, event):
		self.passwd2 = event.GetString()

	def OnButton(self, event):
		if self.username:					
			if not self.passwd1 and not self.passwd2:			
				dialog = wx.MessageDialog(self, 'Please Enter Passwords', 'Warning',
				       wx.OK | wx.ICON_INFORMATION)
				dialog.ShowModal()
				dialog.Destroy()
			elif self.passwd1 != self.passwd2:
				dialog = wx.MessageDialog(self, 'Passwords do not match', 'Warning',
				       wx.OK | wx.ICON_INFORMATION)
				dialog.ShowModal()
				dialog.Destroy()
			else:
				import md5
				hash = md5.new()
				hash.update(self.passwd1)
				self.passwd = hash.hexdigest()
				self.EndModal(wx.ID_OK)
		else:
			dialog = wx.MessageDialog(self, 'Please Enter Username', 'Warning',
				       wx.OK | wx.ICON_INFORMATION)
			dialog.ShowModal()
			dialog.Destroy()
			
	def GetPasswd(self):
		return self.passwd
	
	def GetUsername(self):
		return self.username
		
#----------------------------------------------------------------------------

class EditPasswd(wx.Dialog):
	def __init__(
			self, parent, id, title, size = wx.DefaultSize, pos = wx.DefaultPosition, 
			style = wx.DEFAULT_DIALOG_STYLE
			):
		pre = wx.PreDialog()
		pre.Create(parent, id, title, pos, size, style)

		self.PostCreate(pre)
		self.passwd = None
		self.passwd1 = None
		self.passwd2 = None
		
		self.passwdText1 = wx.StaticText(self, -1, "New Password", size = (120, -1))
		self.passwdText2 = wx.StaticText(self, -1, "Retype Password", size = (120, -1))
		self.passwdCtrl1 = wx.TextCtrl(self, -1, "", size = (150, -1), style = wx.TE_PASSWORD)
		self.passwdCtrl2 = wx.TextCtrl(self, -1, "", size = (150, -1), style = wx.TE_PASSWORD)
		
		self.line = wx.StaticLine(self, -1, style = wx.LI_HORIZONTAL)
		self.button = wx.Button(self, wx.ID_OK, "OK")
		self.button.SetDefault()

		self.sizer = wx.BoxSizer(wx.VERTICAL)
		self.hSizer = wx.BoxSizer(wx.HORIZONTAL)
		self.hSizer.Add(self.passwdText1, 0, wx.ALIGN_CENTRE | wx.ALL, 4)
		self.hSizer.Add(self.passwdCtrl1, 0, wx.ALIGN_CENTRE | wx.BOTTOM | wx.LEFT | wx.RIGHT, 4)
		self.sizer.Add(self.hSizer, 0, wx.ALIGN_CENTRE | wx.ALL, 1)

		self.hSizer = wx.BoxSizer(wx.HORIZONTAL)
		self.hSizer.Add(self.passwdText2, 0, wx.ALIGN_CENTRE | wx.ALL, 4)
		self.hSizer.Add(self.passwdCtrl2, 0, wx.ALIGN_CENTRE | wx.BOTTOM | wx.LEFT | wx.RIGHT, 4)
		self.sizer.Add(self.hSizer, 0, wx.ALIGN_CENTRE | wx.ALL, 1)

		self.sizer.Add(self.line, 0, wx.EXPAND | wx.ALIGN_CENTER_VERTICAL | wx.LEFT | wx.RIGHT, 5)
		self.sizer.Add(self.button, 0, wx.ALIGN_RIGHT | wx.ALL, 5)
		
		self.SetSizer(self.sizer)
		self.SetAutoLayout(True)
		self.sizer.Fit(self)

		self.Bind(wx.EVT_TEXT, self.OnPasswd1, self.passwdCtrl1)
		self.Bind(wx.EVT_TEXT, self.OnPasswd2, self.passwdCtrl2)
		self.Bind(wx.EVT_BUTTON, self.OnButton, self.button, id = wx.ID_OK)

	def OnPasswd1(self, event):
		self.passwd1 = event.GetString()

	def OnPasswd2(self, event):
		self.passwd2 = event.GetString()

	def OnButton(self, event):
		if not self.passwd1 and not self.passwd2:			
			dialog = wx.MessageDialog(self, 'Please Enter Passwords', 'Warning',
                               wx.OK | wx.ICON_INFORMATION)
			dialog.ShowModal()
			dialog.Destroy()
		elif self.passwd1 != self.passwd2:
			dialog = wx.MessageDialog(self, 'Passwords do not match', 'Warning',
                               wx.OK | wx.ICON_INFORMATION)
			dialog.ShowModal()
			dialog.Destroy()
		else:
			import md5
			hash = md5.new()
			hash.update(self.passwd1)
			self.passwd = hash.hexdigest()
			self.EndModal(wx.ID_OK)
			
	def GetPasswd(self):
		return self.passwd
		

#----------------------------------------------------------------------------

class AuthTable(wx.ListCtrl, listmix.ListCtrlAutoWidthMixin):
	def __init__(self, parent, id, pos = wx.DefaultPosition,
				 size = wx.DefaultSize, style = wx.LC_REPORT
								 | wx.BORDER_NONE
								 | wx.LC_EDIT_LABELS
                                                                 | wx.LC_SINGLE_SEL 
								 | wx.LC_SORT_ASCENDING):
		wx.ListCtrl.__init__(self, parent, id, pos, size, style)
		listmix.ListCtrlAutoWidthMixin.__init__(self)

		self.frame = parent.frame
		self.currentItem = None

		self.PopulateList(self.frame.auth_table)
		
		self.Bind(wx.EVT_LIST_BEGIN_LABEL_EDIT, self.OnBeginEdit)
		self.Bind(wx.EVT_LIST_END_LABEL_EDIT, self.OnEndEdit)
		self.Bind(wx.EVT_LIST_ITEM_SELECTED, self.OnItemSelected)
		self.Bind(wx.EVT_COMMAND_RIGHT_CLICK, self.OnRightClick)
		self.Bind(wx.EVT_RIGHT_UP, self.OnRightClick)
		self.Bind(wx.EVT_RIGHT_DOWN, self.OnRightDown)

	def OnBeginEdit(self, event):
		event.Allow()

	def OnEndEdit(self, event):
		user = event.GetText()

		if user:
			self.authTable[self.GetItemData(self.currentItem)][0] = user

	def OnRightDown(self, event):
		self.x = event.GetX()
		self.y = event.GetY()
		item, flags = self.HitTest((self.x, self.y))

		if flags & wx.LIST_HITTEST_ONITEM:
			self.Select(item)
			self.selected = True
		else:
			self.selected = False

		event.Skip()

	def OnRightClick(self, event):
	        if not hasattr(self, "popupEditUser"):
			self.popupEditUsername = wx.NewId()
			self.popupEditPasswd = wx.NewId()
			self.popupRemoveUser = wx.NewId()
			self.popupAddUser = wx.NewId()
		
			self.Bind(wx.EVT_MENU, self.OnPopupEditUsername, id = self.popupEditUsername)
			self.Bind(wx.EVT_MENU, self.OnPopupEditPasswd, id = self.popupEditPasswd)
			self.Bind(wx.EVT_MENU, self.OnPopupRemoveUser, id = self.popupRemoveUser)
			self.Bind(wx.EVT_MENU, self.OnPopupAddUser, id = self.popupAddUser)

		menu = wx.Menu()

		if self.selected:
			menu.Append(self.popupEditUsername, "Edit Username")
			menu.Append(self.popupEditPasswd, "Edit Password")
			menu.Append(self.popupRemoveUser, "Remove User")
		else:
			menu.Append(self.popupAddUser, "Add User")

		self.PopupMenu(menu, (self.x, self.y))
		menu.Destroy()

	def OnPopupEditUsername(self, event):
		self.EditLabel(self.currentItem)
		
	def OnPopupEditPasswd(self, event):
		dialog = EditPasswd(self, -1, "Edit Password")

		if dialog.ShowModal() == wx.ID_OK:
			passwd = dialog.GetPasswd()
			self.authTable[self.GetItemData(self.currentItem)][1] = passwd

		dialog.Destroy()

	def OnPopupRemoveUser(self, event):
		del self.authTable[self.GetItemData(self.currentItem)]
		self.PopulateList(self.authTable, True)
	
	def OnPopupAddUser(self, event):
		dialog = AddUser(self, -1, "Add User")

		if dialog.ShowModal() == wx.ID_OK:
			user = dialog.GetUsername()
			passwd = dialog.GetPasswd()
			self.authTable.append([user, passwd])
			self.PopulateList(self.authTable, True)

		dialog.Destroy()

	def OnItemSelected(self, event):
		self.currentItem = event.m_itemIndex
		self.GetItemText(self.currentItem)
		event.Skip()

	def PopulateList(self, authTable, renew = False):
		if not renew:
			self.InsertColumn(0, "User")
			self.SetColumnWidth(0, wx.LIST_AUTOSIZE_USEHEADER)

			self.authTable = []

			for user, passwd in authTable.items():
				self.authTable.append([user, passwd])

		self.DeleteAllItems()

		for key in range(len(self.authTable)):
			index = self.InsertStringItem(sys.maxint, self.authTable[key][0])
			self.SetItemData(index, key)
	
#----------------------------------------------------------------------------

class ACLTable(wx.ListCtrl, listmix.ListCtrlAutoWidthMixin):
	def __init__(self, parent, id, pos = wx.DefaultPosition,
				 size = wx.DefaultSize, style = wx.LC_REPORT
								 | wx.BORDER_NONE
								 | wx.LC_EDIT_LABELS
                                                                 | wx.LC_SINGLE_SEL 
								 | wx.LC_SORT_ASCENDING):
		wx.ListCtrl.__init__(self, parent, id, pos, size, style)
		listmix.ListCtrlAutoWidthMixin.__init__(self)

		self.frame = parent.frame
		self.currentItem = None

		self.PopulateList(self.frame.acls)
		
		self.Bind(wx.EVT_LIST_BEGIN_LABEL_EDIT, self.OnBeginEdit)
		self.Bind(wx.EVT_LIST_END_LABEL_EDIT, self.OnEndEdit)
		self.Bind(wx.EVT_LIST_ITEM_SELECTED, self.OnItemSelected)
		self.Bind(wx.EVT_COMMAND_RIGHT_CLICK, self.OnRightClick)
		self.Bind(wx.EVT_RIGHT_UP, self.OnRightClick)
		self.Bind(wx.EVT_RIGHT_DOWN, self.OnRightDown)

	def OnBeginEdit(self, event):
		event.Allow()

	def OnEndEdit(self, event):
		addr = event.GetText()

		if addr:			
			if '/' in addr:
				addr, masklen = string.split(addr, '/')
			else:
				masklen = str(32)

			self.aclTable[self.GetItemData(self.currentItem)] = [addr, masklen]

	def OnRightDown(self, event):
		self.x = event.GetX()
		self.y = event.GetY()
		item, flags = self.HitTest((self.x, self.y))

		if flags & wx.LIST_HITTEST_ONITEM:
			self.Select(item)
			self.selected = True
		else:
			self.selected = False

		event.Skip()

	def OnRightClick(self, event):
	        if not hasattr(self, "popupEditAddress"):
			self.popupEditAddress = wx.NewId()
			self.popupRemoveAddress = wx.NewId()
			self.popupAddAddress = wx.NewId()
		
			self.Bind(wx.EVT_MENU, self.OnPopupEditAddress, id = self.popupEditAddress)
			self.Bind(wx.EVT_MENU, self.OnPopupRemoveAddress, id = self.popupRemoveAddress)
			self.Bind(wx.EVT_MENU, self.OnPopupAddAddress, id = self.popupAddAddress)

		menu = wx.Menu()

		if self.selected:
			menu.Append(self.popupEditAddress, "Edit Address")
			menu.Append(self.popupRemoveAddress, "Remove Addresss")
		else:
			menu.Append(self.popupAddAddress, "Add Address")

		self.PopupMenu(menu, (self.x, self.y))
		menu.Destroy()

	def OnPopupEditAddress(self, event):
		self.EditLabel(self.currentItem)
		
	def OnPopupRemoveAddress(self, event):
		del self.aclTable[self.GetItemData(self.currentItem)]		
		self.PopulateList(self.aclTable, True)
	
	def OnPopupAddAddress(self, event):
		dialog = AddAddress(self, -1, "Add Address")
		
		if dialog.ShowModal() == wx.ID_OK:
			addr = dialog.GetAddress()

			if addr:			
				if '/' in addr:
					addr, masklen = string.split(addr, '/')					
				else:
					masklen = str(32)

				self.aclTable.append([addr, masklen])
				self.PopulateList(self.aclTable, True)

		dialog.Destroy()

	def OnItemSelected(self, event):
		self.currentItem = event.m_itemIndex
		self.GetItemText(self.currentItem)
		event.Skip()

	def PopulateList(self, aclTable, renew = False):
		if not renew:
			self.InsertColumn(0, "Address")
			self.SetColumnWidth(0, wx.LIST_AUTOSIZE_USEHEADER)

			self.aclTable = []

			for addr, masklen in aclTable:
				self.aclTable.append([addr, masklen])

		self.DeleteAllItems()

		for key in range(len(self.aclTable)):
			index = self.InsertStringItem(sys.maxint, string.join(self.aclTable[key], "/"))
			self.SetItemData(index, key)
		
#----------------------------------------------------------------------------

class Advanced(wx.Dialog):
	def __init__(
			self, parent, id, title, size = wx.DefaultSize, pos = wx.DefaultPosition, 
			style = wx.DEFAULT_DIALOG_STYLE
			):
		pre = wx.PreDialog()
		pre.Create(parent, id, title, pos, size, style)

		self.PostCreate(pre)

		self.frame = parent.frame
		self.maxConnection = self.frame.max_connection
		self.readSize = self.frame.read_size
		self.maxBuffer = self.frame.max_buffer
		self.sendTimeout = self.frame.send_timeout
		self.shuffle = self.frame.shuffle
		self.recursive = self.frame.recursive
		self.templateFile = os.path.realpath(os.path.join(self.frame.template_dir, self.frame.template))

		self.maxConnectionText = wx.StaticText(self, -1, "Max Connection", size = (100, -1))
		self.readSizeText = wx.StaticText(self, -1, "Read Size", size = (100, -1))
		self.authText = wx.StaticText(self, -1, "Authentication")

		self.maxConnectionCtrl = wx.lib.intctrl.IntCtrl(self, -1, self.maxConnection, size = (150, -1))
		self.readSizeCtrl = wx.lib.intctrl.IntCtrl(self, -1, self.readSize, size = (150, -1))
		self.authCtrl = AuthTable(self, -1, size = (110, 130))
		self.aclCtrl = ACLTable(self, -1)

		self.line = wx.StaticLine(self, -1, style = wx.LI_HORIZONTAL)
		self.button = wx.Button(self, wx.ID_OK, "OK")
		self.button.SetDefault()

		self.sizer = wx.BoxSizer(wx.VERTICAL)
		self.hSizer = wx.BoxSizer(wx.HORIZONTAL)

		self.hSizer.Add(self.maxConnectionText, 0, wx.ALIGN_CENTRE | wx.ALL, 4)
		self.hSizer.Add(self.maxConnectionCtrl, 0, wx.ALIGN_CENTRE | wx.RIGHT | wx.BOTTOM | wx.TOP, 4)
		self.sizer.Add(self.hSizer, 0, wx.ALIGN_CENTER_VERTICAL | wx.ALL, 1)

		self.hSizer = wx.BoxSizer(wx.HORIZONTAL)
		self.hSizer.Add(self.readSizeText, 0, wx.ALIGN_CENTRE | wx.ALL, 4)
		self.hSizer.Add(self.readSizeCtrl, 0, wx.ALIGN_CENTRE | wx.RIGHT | wx.BOTTOM | wx.TOP, 4)
		self.sizer.Add(self.hSizer, 0, wx.ALIGN_CENTER_VERTICAL | wx.ALL, 1)
		
		if self.frame.server_mode == "web":
			self.templateText = wx.StaticText(self, -1, "Template", size = (100, -1))
			self.templateCtrl = wx.TextCtrl(self, -1, size = (150, -1))
			self.templateButton = wx.Button(self, -1, "Browse...")
			self.templateCtrl.SetValue(self.templateFile)

			self.hSizer = wx.BoxSizer(wx.HORIZONTAL)
			self.hSizer.Add(self.templateText, 0, wx.ALIGN_CENTRE | wx.ALL, 4)
			self.hSizer.Add(self.templateCtrl, 0, wx.ALIGN_CENTRE | wx.BOTTOM | wx.TOP, 4)
			self.hSizer.Add(self.templateButton, 0, wx.ALIGN_CENTRE | wx.LEFT | wx.RIGHT | wx.BOTTOM | wx.TOP, 4)
			self.sizer.Add(self.hSizer, 0, wx.ALIGN_CENTER_VERTICAL | wx.ALL, 1)

			self.Bind(wx.EVT_BUTTON, self.OnFileDialog, self.templateButton)
			self.Bind(wx.EVT_TEXT, self.OnTemplateCtrl, self.templateCtrl)
		else:
			self.maxBufferText = wx.StaticText(self, -1, "Max Buffer", size = (100, -1))
			self.sendTimeoutText = wx.StaticText(self, -1, "Send Timeout", size = (100, -1))

			self.sendTimeoutCtrl = wx.lib.intctrl.IntCtrl(self, -1, self.frame.send_timeout, size = (150, -1))
			self.maxBufferCtrl = wx.lib.intctrl.IntCtrl(self, -1, self.frame.max_buffer, size = (150, -1))
			self.shuffleCtrl = wx.CheckBox(self, -1, "Shuffle", style = wx.NO_BORDER | wx.ALIGN_RIGHT)
			self.recursiveCtrl = wx.CheckBox(self, -1, "Recursive", style = wx.NO_BORDER | wx.ALIGN_RIGHT)

			self.shuffleCtrl.SetValue(self.shuffle)
			self.recursiveCtrl.SetValue(self.recursive)

			self.hSizer = wx.BoxSizer(wx.HORIZONTAL)
			self.hSizer.Add(self.maxBufferText, 0, wx.ALIGN_CENTRE | wx.ALL, 4)
			self.hSizer.Add(self.maxBufferCtrl, 0, wx.ALIGN_CENTRE | wx.RIGHT | wx.BOTTOM | wx.TOP, 4)
			self.sizer.Add(self.hSizer, 0, wx.ALIGN_CENTER_VERTICAL | wx.ALL, 1)

			self.hSizer = wx.BoxSizer(wx.HORIZONTAL)
			self.hSizer.Add(self.sendTimeoutText, 0, wx.ALIGN_CENTRE | wx.ALL, 4)
			self.hSizer.Add(self.sendTimeoutCtrl, 0, wx.ALIGN_CENTRE | wx.RIGHT | wx.BOTTOM | wx.TOP, 4)
			self.sizer.Add(self.hSizer, 0, wx.ALIGN_CENTER_VERTICAL | wx.ALL, 1)

			self.sizer.Add(self.shuffleCtrl, 0, wx.ALIGN_RIGHT | wx.ALL, 5)
			self.sizer.Add(self.recursiveCtrl, 0, wx.ALIGN_RIGHT | wx.RIGHT | wx.LEFT | wx.BOTTOM, 5)
			
			self.Bind(wx.lib.intctrl.EVT_INT, self.OnSendTimeout, self.sendTimeoutCtrl)
			self.Bind(wx.lib.intctrl.EVT_INT, self.OnMaxBuffer, self.maxBufferCtrl)
			self.Bind(wx.EVT_CHECKBOX, self.OnShuffle, self.shuffleCtrl)
			self.Bind(wx.EVT_CHECKBOX, self.OnRecursive, self.recursiveCtrl)
		
		self.sizer.Add((-1, 20), 0, wx.EXPAND)
		self.sizer.Add(self.authText, 0, wx.ALIGN_LEFT | wx.ALL, 5)

		self.hSizer = wx.BoxSizer(wx.HORIZONTAL)
		self.hSizer.Add(self.authCtrl, 0, wx.EXPAND | wx.ALIGN_CENTRE | wx.BOTTOM | wx.LEFT, 4)
		self.hSizer.Add(self.aclCtrl, 1, wx.EXPAND | wx.ALIGN_CENTRE | wx.BOTTOM | wx.LEFT | wx.RIGHT, 4)
		self.sizer.Add(self.hSizer, 1, wx.EXPAND | wx.ALIGN_CENTRE | wx.ALL, 1)
		self.sizer.Add(self.line, 0, wx.EXPAND | wx.ALIGN_CENTER_VERTICAL | wx.LEFT | wx.RIGHT, 5)
		self.sizer.Add(self.button, 0, wx.ALIGN_RIGHT | wx.ALL, 5)

		self.SetSizer(self.sizer)
		self.SetAutoLayout(True)
		self.sizer.Fit(self)

		self.Bind(wx.EVT_BUTTON, self.OnButton, self.button, id = wx.ID_OK)
		self.Bind(wx.lib.intctrl.EVT_INT, self.OnMaxConnection, self.maxConnectionCtrl)
		self.Bind(wx.lib.intctrl.EVT_INT, self.OnReadSize, self.readSizeCtrl)

	def OnFileDialog(self, event):
		wildcard = "MOSS Template (*.tmpl)|*.tmpl|" \
			"Compiled MOSS Template (*.tmplc)|*.tmplc|" \
			"All files (*.*)|*.*"

		dialog = wx.FileDialog(self, "Template File", defaultDir = os.getcwd(), 
				defaultFile = "", wildcard = wildcard, style = wx.OPEN | wx.CHANGE_DIR)

		if dialog.ShowModal() == wx.ID_OK:
			self.templateFile = dialog.GetPath()
			self.templateCtrl.SetValue(self.templateFile)
			self.Layout()

		dialog.Destroy()

	def OnMaxConnection(self, event):
		self.maxConnection = event.GetValue()

	def OnReadSize(self, event):
		self.readSize = event.GetValue()
		
	def OnTemplateCtrl(self, event):
		self.templateFile = event.GetString()

	def OnSendTimeout(self, event):
		self.sendTimeout = event.GetValue()

	def OnMaxBuffer(self, event):
		self.maxBuffer = event.GetValue()

	def OnShuffle(self, event):
		if event.IsChecked():
			self.shuffle = 1
		else:
			self.shuffle = 0

	def OnRecursive(self, event):
		if event.IsChecked():
			self.recursive = 1
		else:
			self.recursive = 0

	def OnButton(self, event):
		self.frame.max_connection = self.maxConnection
		self.frame.read_size = self.readSize
		self.frame.max_buffer = self.maxBuffer
		self.frame.send_timeout = self.sendTimeout
		self.frame.shuffle = self.shuffle
		self.frame.recursive = self.recursive
		self.frame.template_dir, self.frame.template = os.path.split(self.templateFile)

		self.frame.auth_table = {}
		self.frame.acls = []

		for key in range(len(self.authCtrl.authTable)):
			self.frame.auth_table[self.authCtrl.authTable[key][0]] = self.authCtrl.authTable[key][1]

		for key in range(len(self.aclCtrl.aclTable)):
			self.frame.acls.append(self.aclCtrl.aclTable[key])		

		self.EndModal(wx.ID_OK)
		
#----------------------------------------------------------------------------

class AddSource(wx.Dialog):
	def __init__(
			self, parent, id, title, size = wx.DefaultSize, pos = wx.DefaultPosition, 
			style = wx.DEFAULT_DIALOG_STYLE
			):
		pre = wx.PreDialog()
		pre.Create(parent, id, title, pos, size, style)

		self.PostCreate(pre)

		self.label = None
		self.path = None
		self.labelText = wx.StaticText(self, -1, "Label", size = (50, -1))
		self.dirText = wx.StaticText(self, -1, "Directory", size = (50, -1))

		self.labelCtrl = wx.TextCtrl(self, -1, size = (100, -1))
		self.dirCtrl = wx.TextCtrl(self, -1, size = (170, -1))
		self.dirButton = wx.Button(self, -1, "Browse...") 
		
		self.line = wx.StaticLine(self, -1, style = wx.LI_HORIZONTAL)
		self.button = wx.Button(self, wx.ID_OK, "OK")
		self.button.SetDefault()

		self.sizer = wx.BoxSizer(wx.VERTICAL)
		self.hSizer = wx.BoxSizer(wx.HORIZONTAL)
		
		self.hSizer.Add(self.labelText, 0, wx.ALIGN_CENTRE | wx.ALL, 4)
		self.hSizer.Add(self.labelCtrl, 0, wx.ALIGN_CENTRE | wx.RIGHT | wx.BOTTOM | wx.TOP, 4)
		self.sizer.Add(self.hSizer, 0, wx.ALIGN_CENTER_VERTICAL | wx.ALL, 1)

		self.hSizer = wx.BoxSizer(wx.HORIZONTAL)
		
		self.hSizer.Add(self.dirText, 0, wx.ALIGN_CENTRE | wx.ALL, 4)
		self.hSizer.Add(self.dirCtrl, 0, wx.ALIGN_CENTRE | wx.RIGHT | wx.BOTTOM | wx.TOP, 4)
		self.hSizer.Add(self.dirButton, 0, wx.ALIGN_CENTRE | wx.RIGHT | wx.BOTTOM | wx.TOP, 4)

		self.sizer.Add(self.hSizer, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT | wx.RIGHT | wx.BOTTOM, 1)
		self.sizer.Add(self.line, 0, wx.EXPAND | wx.ALIGN_CENTER_VERTICAL | wx.LEFT | wx.RIGHT, 5)
		self.sizer.Add(self.button, 0, wx.ALIGN_RIGHT | wx.ALL, 5)

		self.SetSizer(self.sizer)
		self.SetAutoLayout(True)
		self.sizer.Fit(self)

		self.Bind(wx.EVT_TEXT, self.OnLabelCtrl, self.labelCtrl)
		self.Bind(wx.EVT_TEXT, self.OnDirCtrl, self.dirCtrl)
		self.Bind(wx.EVT_BUTTON, self.OnDirDialog, self.dirButton)
		self.Bind(wx.EVT_BUTTON, self.OnButton, self.button, id = wx.ID_OK)

	def OnDirDialog(self, event):
		dialog = wx.DirDialog(self, "Source Directory",
                          style = wx.DD_DEFAULT_STYLE)

		if dialog.ShowModal() == wx.ID_OK:
			self.path = dialog.GetPath()
			self.dirCtrl.SetValue(self.path)
			self.Layout()

		dialog.Destroy()

	def OnButton(self, event):
		if self.label and self.path:
			self.EndModal(wx.ID_OK)
		else:
			dialog = wx.MessageDialog(self, 'Please Enter Values', 'Warning',
				       wx.OK | wx.ICON_INFORMATION)
			dialog.ShowModal()
			dialog.Destroy()

	def OnLabelCtrl(self, event):
		self.label = event.GetString()

	def OnDirCtrl(self, event):
		self.path = event.GetString()

	def GetValue(self):
		return self.label, os.path.realpath(self.path)
		
#----------------------------------------------------------------------------

class MyFrame(wx.Frame):
	def __init__(self, parent, id, title, pos = wx.DefaultPosition,
			size = wx.DefaultSize, style = wx.DEFAULT_FRAME_STYLE & ~(wx.RESIZE_BORDER | wx.RESIZE_BOX | wx.MAXIMIZE_BOX)):
		wx.Frame.__init__(self, parent, id, title, pos, size, style)
		self.stayOnTop = False
		self.icon = wx.Icon(os.path.join('pixmaps', 'moss.ico'), wx.BITMAP_TYPE_ICO) 
		self.SetIcon(self.icon)

		if wx.Platform != '__WXMAC__':
			if wx.Platform == '__WXGTK__':
				dim = 22
			else:
				dim = 16

			self.icon = wx.IconFromBitmap(
				wx.Image(os.path.join('pixmaps', 'moss.ico')).Scale(dim, dim).ConvertToBitmap())
			self.tbicon = wx.TaskBarIcon()
			self.tbicon.SetIcon(self.icon, __package__)

		self.config = ConfigParser.ConfigParser()
		config_defaults = self.config.defaults()
		
		config_defaults["server-mode"] = "web"
		config_defaults["port"] = "8162"
		config_defaults["hostname"] = ""
		config_defaults["log"] = "moss.log"
		config_defaults["max-connection"] = "10"
		config_defaults["read-size"] = "8192"
		config_defaults["stream-type"] = "mp3"
		config_defaults["max-buffer"] = "81920"
		config_defaults["send-timeout"] = "3"
		config_defaults["shuffle"] = "0"
		config_defaults["recursive"] = "1"
		config_defaults["template"] = "default.tmpl"
		config_defaults["template-dir"] = "templates"

		self.config_file = os.path.realpath(os.path.join("conf", "moss.conf"))

		if os.path.isfile(self.config_file):
			self.config.read(self.config_file)
		else:
			self.config.add_section("server")
			self.config.add_section("sources")
			self.config.add_section("acl")
			self.config.add_section("extra")
		
		self.server_mode = self.config.get("server", "server-mode")
		self.port = self.config.getint("server", "port")
		self.hostname = self.config.get("server", "hostname")
		self.log = os.path.realpath(self.config.get("server", "log"))
		self.max_connection = self.config.getint("server", "max-connection")
		self.read_size = self.config.getint("server", "read-size")
		self.stream_type = self.config.get("server", "stream-type")
		self.max_buffer = self.config.getint("server", "max-buffer")
		self.send_timeout = self.config.getint("server", "send-timeout")
		self.shuffle = self.config.getint("server", "shuffle")
		self.recursive = self.config.getint("server", "recursive")
		self.template = self.config.get("extra", "template")
		self.template_dir = os.path.realpath(self.config.get("extra", "template-dir"))

		self.acls = []

		try:
			allowed = re.split(r"[\s\n,]+", self.config.get("acl", "allow"))
		except ConfigParser.NoOptionError:
			allowed = []

		for addr in allowed:
			if '/' in addr:
				addr, masklen = string.split(addr, '/')				
			else:
				masklen = str(32)

			self.acls.append((addr, masklen))

		try:
			self.auth_file = os.path.realpath(self.config.get("acl", "auth-file"))
			
			if os.path.isfile(self.auth_file):
				fp = open(self.auth_file, "r")

				try:
					self.auth_table = {}

					for pair in fp.readlines():
						user, passwd = string.split(pair[:-1], ':')
						self.auth_table[user] = passwd
				except ValueError:
					pass

				fp.close()
		except ConfigParser.NoOptionError:
			self.auth_table = {}
			self.auth_file = os.path.realpath(os.path.join("conf", "acl.txt"))
		except:
			pass
				
		self.dirs = []

		for option in self.config.options("sources"):
			if option[:10] == "source-dir":
				self.dirs.append((int(option[10:]), self.config.get("sources", option)))

                if not self.dirs:
                    self.dirs.append((0, string.join([os.path.realpath("images"), "Images"], "::")))

		self.panel = MyPanel(self, -1)
		self.panel.sizer.Fit(self)

		self.Bind(wx.EVT_TOOL, self.OnExit, id = ID_EXIT)
		self.Bind(wx.EVT_CLOSE, self.OnTaskHide)
		self.tbicon.Bind(wx.EVT_MENU, self.OnAbout, id = ID_ABOUT)
		self.tbicon.Bind(wx.EVT_MENU, self.OnTaskShow, id = ID_SHOW)
		self.tbicon.Bind(wx.EVT_MENU, self.OnTaskHide, id = ID_HIDE)
		self.tbicon.Bind(wx.EVT_MENU, self.OnExit, id = ID_EXIT)
		self.tbicon.Bind(wx.EVT_TASKBAR_RIGHT_UP, self.OnTaskMenu)
		self.tbicon.Bind(wx.EVT_TASKBAR_LEFT_DCLICK, self.OnCheckShowHideToggle)
		
	def OnExit(self,event):
		busy = wx.BusyInfo("One moment please...")
		wx.Yield()

		if self.panel.thread:
			self.panel.thread.stop()

		running = True

		while running:
			running = False

			if self.panel.thread:
				running = running + self.panel.thread.IsRunning()

			time.sleep(0.1)

		self.Destroy()

	def OnTaskShow(self,event):
		self.Show(True)
		 
	def OnTaskHide(self,event):
		self.Show(False)

	def OnCheckShowHideToggle(self,event):
		if self.IsShown():
			self.Show(False)
		else:
			self.Show(True)

	def OnTaskMenu(self, event):
		menu = wx.Menu()

		menu.Append(ID_ABOUT, "About MOSS")

		if self.IsShown():
			menu.Append(ID_HIDE, "Hide Window")
		else:
			menu.Append(ID_SHOW, "Show Window")

		menu.AppendSeparator()
		menu.Append(ID_EXIT, "Exit Program")
 
		self.tbicon.PopupMenu(menu)
		menu.Destroy()
		
	def OnAbout(self, event):
		dialog = MyAbout(self, -1, "About MOSS")
		dialog.CenterOnScreen()
		dialog.ShowModal()
		dialog.Destroy()

#----------------------------------------------------------------------------

class MyAbout(wx.Dialog):
	def __init__(
			self, parent, ID, title, size = wx.DefaultSize, pos = wx.DefaultPosition, 
			style = wx.DEFAULT_DIALOG_STYLE
			):
		pre = wx.PreDialog()
		pre.Create(parent, ID, title, pos, size, style)

		self.PostCreate(pre)

		self.bitmap = wx.Image(os.path.join('pixmaps', 'moss.bmp'), wx.BITMAP_TYPE_BMP).ConvertToBitmap()
		self.logo = wx.StaticBitmap(self, -1, self.bitmap, size = (self.bitmap.GetWidth(), self.bitmap.GetHeight()))
		self.line = wx.StaticLine(self, -1, style = wx.LI_HORIZONTAL)
		self.button = wx.Button(self, wx.ID_OK, "OK")
		self.button.SetDefault()

		self.sizer = wx.BoxSizer(wx.VERTICAL)
		self.sizer.Add(self.logo, 0, wx.ALIGN_CENTRE | wx.ALL, 5)
		self.sizer.Add(self.line, 0, wx.EXPAND | wx.ALIGN_CENTER_VERTICAL | wx.LEFT | wx.RIGHT, 5)
		self.sizer.Add(self.button, 0, wx.ALIGN_CENTRE | wx.ALL, 5)

		self.SetSizer(self.sizer)
		self.SetAutoLayout(True)
		self.sizer.Fit(self)

#----------------------------------------------------------------------------

class MyApp(wx.App):
	def OnInit(self):
		wx.InitAllImageHandlers()

		win = MyFrame(None, -1, __package__)
		win.Show(True)
		self.SetTopWindow(win)

		return True

#----------------------------------------------------------------------------

def main():
	instance = wx.SingleInstanceChecker(__package__)

	if instance.IsAnotherRunning():
		sys.exit(1)
	else:
		app = MyApp(0)
		app.MainLoop()

#----------------------------------------------------------------------------

if __name__ == '__main__':
	main()
