#-*-coding:mbcs-*- 
#----------------------------------------------------------------------------
# Project Name: MOSS for Windows
# File Name: moss.py
# Description: Run server.
#
# Created: 2004. 10. 10
# RCS-ID: $Id: moss.py,v 1.8 2004/10/22 01:30:15 myunggoni Exp $
# Copyright: (c) 2004 by myunggoni
# License: GNU General Public License
# Author: Myung-Gon Park <myunggoni@users.kldp.net>
#----------------------------------------------------------------------------

__package__ = "MOSS for Windows"
__version__ = "0.1.1"

import ConfigParser
import sys
import getopt
import os
import live
import web

try:
	import ogg.vorbis
	ogg_support = "yes"
except ImportError:
	ogg_support = "no"

class MOSS:
	def __init__(self, config_file, server_mode):
		self.running = False
		self.reconfig = False
		self.config_file = config_file
		self.server_mode = server_mode

		if server_mode == "web":
			self.server = web.Server(config_file)
		else:
			self.server = live.Server(config_file)
		
	
	def run(self):
		self.running = True
		self.server.log_message("Server started with pid %d" % os.getpid())

		if self.server_mode == "web":									
			try:
				while self.running:
					if self.reconfig:
						self.server.log_message("Reloading configuraion file")
						self.server.server_close()
						self.server = web.Server(self.config_file)
						self.reconfig = False

					self.server.handle_request()

				self.server.log_message("Now taking down the server...")
			
			except KeyboardInterrupt:
				self.stop()
				self.server.log_message("KeyboardInterrupt occured")
				self.server.log_message("Please wait while the remaining streams finish...")
			except:
				self.stop()
				self.server.log_message("Please wait while the remaining streams finish...")
		else:
			try:
				while self.running:
					if self.reconfig:
						self.server.log_message("Reloading configuration file")
						self.server.server_close()
						self.server = live.Server(self.config_file)
						self.reconfig = False

					self.server.serve_forever()
			
				self.server.log_message("Now taking down the server...")
			
			except KeyboardInterrupt:
				self.stop()
				self.server.log_message("KeyboardInterrupt occured")
			except:
				self.stop()
				self.server.log_message("Please wait while the remaining streams finish...")

	def stop(self):
		self.running = False
		self.server.server_close()

	def restart(self):
		self.reconfig = True
		self.server.server_close()		

	def IsRunning(self):
		return self.running		

def print_usage():
	print "Usage: %s [-c <filename>]" % os.path.basename(sys.argv[0])
	print "       -h     --help         Display this usage information"
	print "       -c     --config-file  Specify the config file (default: %s)" % os.path.join("conf", "moss.conf")

def main():
	try:
		opts, args = getopt.getopt(sys.argv[1:], "hc:", ["help", "config-file"])
	except getopt.GetoptError:
		print_usage()
		sys.exit(1)

	config_file = os.path.join("conf", "moss.conf")

	for opt, arg in opts:
		if opt in ("-h", "--help"):
			print_usage()
			sys.exit(0)
				
		if opt in ("-c", "--config-file"):
			config_file = arg
			
			if not os.path.isfile(config_file):
				print "No such configuration file"
				print_usage()
				sys.exit(1)

	if os.path.isfile(config_file):
		config = ConfigParser.ConfigParser()
		config_defaults = config.defaults()
		config_defaults["server-mode"] = "web"
		config.read(config_file)
		server_mode = config.get("server", "server-mode")
	
		if not server_mode in ("web", "live"):
			print "Invalid server mode"
			sys.exit(1)
			
	else:
		print "Configuration file not found"
		sys.exit(1)

	print "Entering in %s mode" % server_mode

	if ogg_support == "yes":
		print "Ogg vorbis support enabled"
	else:
		print "Ogg vorbis support disabled"
		print "To enable it you will need to install the 'pyogg' and the 'pyvorbis' modules"

	server = MOSS(config_file, server_mode)
	server.run()
			
if __name__ == "__main__":
	main()
